@@ -7,9 +7,21 @@ use std::iter::Sum;
 use std::ops::{Add, AddAssign};
 
 /// Implementation styles for WebAssembly linear memory.
-#[derive(Debug, Clone, Copy, PartialEq, Eq, Hash, RkyvSerialize, RkyvDeserialize, Archive)]
+#[derive(
+    Debug,
+    Clone,
+    Copy,
+    PartialEq,
+    Eq,
+    Hash,
+    RkyvSerialize,
+    RkyvDeserialize,
+    Archive,
+    rkyv::CheckBytes,
+)]
 #[cfg_attr(feature = "enable-serde", derive(Serialize, Deserialize))]
 #[archive(as = "Self")]
+#[repr(u8)]
 pub enum MemoryStyle {
     /// The actual memory can be resized and moved.
     Dynamic {
@@ -59,6 +71,8 @@ pub unsafe trait MemorySize: Copy {
         + PartialOrd<Self::Offset>
         + Clone
         + Copy
+        + Sync
+        + Send
         + ValueType
         + Into<u64>
         + From<u32>
@@ -68,15 +82,18 @@ pub unsafe trait MemorySize: Copy {
         + TryFrom<u32>
         + TryFrom<u16>
         + TryFrom<u8>
+        + TryFrom<i32>
         + TryInto<usize>
         + TryInto<u64>
         + TryInto<u32>
         + TryInto<u16>
         + TryInto<u8>
+        + TryInto<i32>
         + TryFrom<usize>
         + Add<Self::Offset>
         + Sum<Self::Offset>
-        + AddAssign<Self::Offset>;
+        + AddAssign<Self::Offset>
+        + 'static;
 
     /// Type used to pass this value as an argument or return value for a Wasm function.
     type Native: super::NativeWasmType;
